%**************************************************************************
%
%   BENCHMARK STUDY ON RELIABILITY ESTIMATION IN HIGHER DIMENSIONS OF
%   STRUCTURAL SYSTEMS
%
%   Sample code for solving Problem 3 using direct Monte Carlo Simulation
%
%   A complete description of this problem can be found in:
%   G. I. Schuëller & H. J. Pradlwarter. Benchmark Study on Reliability 
%   Estimation in Higher Dimensions of Structural Systems - An Overview.
%   Structural Safety 29, 167-182, 2007.
%
%   This code is available at
%   http://mechanik.uibk.ac.at/Benchmark_Reliability/
%
%**************************************************************************
%   BECAUSE THE CODE IS PROVIDED FREE OF CHARGE, IT IS PROVIDED "AS IS" AND
%   WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED. 
%**************************************************************************

clear all;

%% 1.   Data
%1.1.   Data on the model
N_Sim       = input('Number of simulations => ');        %Number of Simulations
n_rv_str    = 25;   %number of structural random variables
n_rv_exc    = 200;  %number of random variables associated with excitation
n_dof       = 5;    %number of degrees of freedom
n_steps     = 2001; %number of time steps for integration
Dt          = 0.01;             %Discrete time step
threshold   = [0.032,0.022];    %threshold level
counter_failure_1   = 0;        %counter - failure event 1
counter_failure_2   = 0;        %counter - failure event 2
% Mean of structural parameters
%     k_i      r_i      m_i     xi_i    s_i/k_i
mean_struct_par= [
    2.4e+07  2.4e+06  2.0e+04  6.0e-02  8.0e-03
    2.1e+07  2.1e+06  2.0e+04  6.0e-02  8.0e-03
    1.8e+07  1.8e+06  2.0e+04  6.0e-02  8.0e-03
    1.5e+07  1.5e+06  2.0e+04  6.0e-02  8.0e-03
    1.2e+07  1.2e+06  2.0e+04  6.0e-02  8.0e-03
    ];
% Std. dev. of structural parameters
%     k_i       r_i     m_i     xi_i    s_i/k_i
stdev_struct_par= [
    2.4e+06  2.4e+05  2.0e+03  1.0e-02  8.0e-04
    2.1e+06  2.1e+05  2.0e+03  1.0e-02  8.0e-04
    1.8e+06  1.8e+05  2.0e+03  1.0e-02  8.0e-04
    1.5e+06  1.5e+05  2.0e+03  1.0e-02  8.0e-04
    1.2e+06  1.2e+05  2.0e+03  1.0e-02  8.0e-04
    ];
%1.2.   Random number generator
seed        = 1;        %seed for random number generator
rand('twister',seed);
randn('state',seed+1);

%% 2.   Direct Monte Carlo Simulation
tic;
for i=1:N_Sim,
    %Generate realization of structural parameters
    z_struct    = randn(n_rv_str,1);    %random numbers for generating realization of structural parameters
    sample_str  = mean_struct_par + ...
        reshape(z_struct,size(mean_struct_par)) .* stdev_struct_par;    %realization structural parameters
    %Generate realization of ground acceleration
    z_acc   = randn(n_rv_exc,1);    %random numbers for generating realization of acceleration
    t       = 0:Dt:(n_steps-1)*Dt;  %time
    h       = [(0:0.01:2)*0.5,ones(1,799),exp(-0.1*(0:0.01:10))];   %envelope function
    vec_1   = (1:2:199).';          %auxiliary vector
    vec_2   = (2:2:200).';          %auxiliary vector
    a(vec_1,1:n_steps)  = 0.1*cos(2*pi*(1+0.1*(0:99).')*t);         %auxiliary vector
    a(vec_2,1:n_steps)  = 0.1*sin(2*pi*(1+0.1*(0:99).')*t);         %auxiliary vector
    sample_acc          = h.*(z_acc'*a);    %realization of acceleration
    %Solve equation of motion
    [x]    = SEM3(n_dof,n_steps,Dt,sample_acc,sample_str);
    %Check whether or not failure has occurred
    counter_failure_1   = counter_failure_1 + (max(abs(x(:,1)))>threshold(1));          %counter - failure event 1
    counter_failure_2   = counter_failure_2 + (max(abs(x(:,5)-x(:,4)))>threshold(2));   %counter - failure event 2
end

%Output
disp([num2str(i) ' samples have been computed in ' num2str(toc) ' [s]']);
disp('Failure probability associated with different threshold levels');
pf1     = counter_failure_1/i;
if pf1>0,
    cov1    = sqrt((1-pf1)/(i*pf1));
else
    cov1    = nan;
end
pf2     = counter_failure_2/i;
if pf2>0,
    cov2    = sqrt((1-pf2)/(i*pf2));
else
    cov2    = nan;
end
disp(['P[F1] = ' num2str(pf1) '; CoV= ' num2str(cov1)]);
disp(['P[F2] = ' num2str(pf2) '; CoV= ' num2str(cov2)]);


return
